package com.taauav.admin.controller;


import com.taauav.common.bean.Response;
import com.taauav.common.constant.PermissionConstants;
import com.taauav.common.constant.SysLevelConstant;
import com.taauav.admin.entity.SysLevel;
import com.taauav.admin.service.ISysLevelService;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Map;

/**
 * <p>
 * 职级管理表 前端控制器
 * </p>
 *
 * @author daixt
 * @since 2019-11-11
 */
@RestController
@RequestMapping("/level")
public class SysLevelController {
    @Autowired
    private ISysLevelService levelService;
    @Resource
    private Response response;

    private static final String controllerName = "level";

    /**
     * 职级列表接口
     * @param map
     * @return
     */
    @PostMapping("/index")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.LIST_PERMISSION)
    public Response index(@RequestBody Map<String, String> map) {
        return levelService.pageList(map);
    }

    /**
     * 编辑-提交接口
     * @param map
     * @return
     */
    @PostMapping(value = "/edit")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response edit(@RequestBody(required = false) SysLevel map) {
        return levelService.edit(map);
    }

    /**
     * 编辑-获取原数据接口
     * @param id
     * @return
     */
    @GetMapping("/getLevelInfo")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response getLevelInfo(Integer id) {
        if (id <= 0) {
            return response.failure("职级id必须为正整数");
        }
        return levelService.getLevelInfo(id);
    }

    /**
     * 添加职级接口
     * @param map
     * @return
     */
    @PostMapping("/add")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.ADD_PERMISSION)
    public Response add(@RequestBody(required = true) SysLevel map) {
        return levelService.edit(map);
    }

    /**
     * 删除职级接口
     * @param map
     * @return
     */
    @PostMapping("/drop")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.REMOVE_PERMISSION)
    public Response drop(@RequestBody(required = true) SysLevel map) {
        if (map.getId() == null || map.getId() <= 0) {
            return response.failure("职级id必须为正整数");
        }
        return levelService.drop(map.getId());
    }

    /**
     * 修改职级状态接口
     * @param map
     * @return
     */
    @PostMapping("/changeStatus")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.STATUS_PERMISSION)
    public Response changeStatus(@RequestBody(required = true) SysLevel map) {
        if (map.getId() == null || map.getId() <= 0) {
            return response.failure("职级id必须为正整数");
        }
        return levelService.changeStatus(map.getId(), map.getStatus());
    }

    /**
     * 获取职级状态
     * @return
     */
    @PostMapping("/getStatus")
    public Response getStatus() {
        return response.success("操作成功", SysLevelConstant.getLevelStatus());
    }
}
