package com.taauav.admin.controller;


import com.taauav.admin.dto.TauvInspectDriverAssignTaskDTO;
import com.taauav.common.bean.Response;
import com.taauav.common.constant.PermissionConstants;
import com.taauav.admin.entity.TauvInspect;
import com.taauav.admin.service.ISysAdminService;
import com.taauav.admin.service.ISysAuthGroupService;
import com.taauav.admin.service.ISysCityService;
import com.taauav.admin.service.ITauvInspectDriverService;
import com.taauav.admin.validate.ExecuteTask;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.math.BigInteger;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * 巡检任务河道表 前端控制器
 * </p>
 *
 * @author dyg
 * @since 2019-11-14
 */
@RestController
@RequestMapping("/inspectDriver")
public class TauvInspectDriverController {
    @Autowired
    private ITauvInspectDriverService iTauvInspectDriverService;
    @Autowired
    private Response response;
    @Autowired
    private ISysCityService cityService;
    @Autowired
    private ISysAdminService adminService;
    @Autowired
    private ISysAuthGroupService authGroupService;

    private static final String controllerName = "inspectDriver";

    /**
     * 任务列表--未分配
     *
     * @param map
     * @return
     */
    @PostMapping("/index")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.LIST_PERMISSION)
    public Response index(@RequestBody(required = false) Map<String, Object> map) {
        Integer[] st = new Integer[]{1};
        if (map == null) {
            map = new HashMap<>();
        }
        map.put("status", st);
        return iTauvInspectDriverService.getPageList(map, "assign");
    }

    /**
     * 任务列表--已分配
     *
     * @param map
     * @return
     */
    @PostMapping("/assignedList")
    @RequiresPermissions(controllerName + ":assignedList")
    public Response assignedList(@RequestBody(required = false) Map<String, Object> map) {
        Integer[] st = new Integer[]{2, 3, 4, 5, 6};
        if (map == null) {
            map = new HashMap<>();
        }
        String status = "status";
        if (map.containsKey(status) && !"".equals(map.get(status))) {
            map.put("status", new Integer[]{Integer.valueOf(map.get(status).toString())});
        } else {
            map.put("status", st);
        }
        return iTauvInspectDriverService.getPageList(map, "assign");
    }

    /**
     * 任务列表
     *
     * @param map
     * @return
     */
    @PostMapping("/list")
    @RequiresPermissions(controllerName + ":list")
    public Response taskList(@RequestBody(required = false) Map<String, Object> map) {
        if (map == null) {
            map = new HashMap<>();
        }
        String status = "status";
        if (map.containsKey(status) && !"".equals(map.get(status))) {
            map.put("status", Integer.valueOf(map.get(status).toString()));
        } else {
            map.put("status", new Integer[]{2, 3, 4, 5, 6});
        }
        return iTauvInspectDriverService.getPageList(map, "list");
    }

    /**
     * 获取待分类任务列表
     *
     * @param inspectDriverIds 待分配任务ID集合
     * @return
     */
    @GetMapping("/getAssignTaskList")
    public Response getAssignTaskList(String inspectDriverIds) {
        return iTauvInspectDriverService.getAssignTaskList(inspectDriverIds);
    }

    /**
     * 分配
     *
     * @param inspectDriverAssignTaskDTO
     * @return
     */
    @RequestMapping("/assignTask")
    @RequiresPermissions(controllerName + ":assign")
    public Response assignTask(@RequestBody TauvInspectDriverAssignTaskDTO inspectDriverAssignTaskDTO) {
        return iTauvInspectDriverService.assignTask(inspectDriverAssignTaskDTO);
    }

    /**
     * 任务详情
     */
    @GetMapping("/detail")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.VIEW_PERMISSION)
    public Response detail(@RequestBody @RequestParam("id") Integer id) {
        Map<String, Object> info = iTauvInspectDriverService.detail(id);
        if (StringUtils.isEmpty(info.get("error"))) {
            return response.success(info);
        } else {
            return response.failure(info.get("error"));
        }
    }

    /**
     * 取消任务
     *
     * @param map
     * @return
     */
    @PostMapping("/cancel")
    @RequiresPermissions(controllerName + ":cancel")
    public Response cancel(@RequestBody Map<String, String> map) {
        return iTauvInspectDriverService.cancel(map);
    }

    /**
     * 执行任务
     *
     * @return
     */
    @PostMapping("/executeTask")
    @RequiresPermissions(controllerName + ":execute")
    public Response executeTask(@RequestBody @Valid ExecuteTask executeTask) {
        return iTauvInspectDriverService.executeTask(executeTask);
    }

    /**
     * 获取打印的数据信息
     *
     * @param id
     * @return
     */
    @GetMapping("/print")
    @RequiresPermissions(controllerName + ":print")
    public Response print(@RequestBody @RequestParam("id") Integer id) {
        return iTauvInspectDriverService.getPrintData(id);
    }

    /**
     * 获取基础数据
     *
     * @return
     */
    @GetMapping("/basicData")
    public Response basicData() {
        Map<String, Object> result = new HashMap<>(4);
        TauvInspect tauvInspect = new TauvInspect();
        //状态数组
        result.put("statusList", tauvInspect.getStatusList());
        //分类数组
        result.put("typeList", tauvInspect.getTypeList());
        //区域数组
        //result.put("areaList",cityService.getCityListByPidAndAdmin(320100));
        result.put("areaList", cityService.getCityList(BigInteger.valueOf(Long.valueOf("320117")), true));
        //飞行小组
        result.put("adminList", adminService.getAdminListByGroupId(6));
        return response.success("操作成功", result);
    }

    /**
     * 判断是否已生成报告
     *
     * @param map
     * @return
     */
    @PostMapping("/existReport")
    public Response existReport(@RequestBody Map<String, Object> map) {
        if (StringUtils.isEmpty(map)) {
            return response.failure("参数不能为空");
        }
        return iTauvInspectDriverService.existReport(map);
    }

    /**
     * 删除任务
     *
     * @param map
     * @return
     */
    @PostMapping("/dropTask")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.REMOVE_PERMISSION)
    public Response dropTask(@RequestBody Map<String, Object> map) {
        return iTauvInspectDriverService.dropTask(map);
    }

    /**
     * 重处理任务
     *
     * @param map
     * @return
     */
    @PostMapping("/reProcess")
    @RequiresPermissions(controllerName + ":reProcess")
    public Response reProcess(@RequestBody Map<String, Object> map) {
        return iTauvInspectDriverService.reProcess(map);
    }
}
