package com.taauav.admin.controller;


import com.taauav.common.bean.Response;
import com.taauav.common.constant.PermissionConstants;
import com.taauav.admin.entity.TauvReportEvaluation;
import com.taauav.admin.service.ITauvReportEvaluationService;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Map;

/**
 * <p>
 * 报告描述(评价)内容管理表 前端控制器
 * </p>
 *
 * @author dyg
 * @since 2019-11-14
 */
@RestController
@RequestMapping("/evaluation")
public class TauvReportEvaluationController {
    @Autowired
    private ITauvReportEvaluationService iTauvReportEvaluationService;
    @Autowired
    private Response response;

    private static final String controllerName = "evaluation";

    /**
     * 评价数据列表
     * @param map
     * @return
     */
    @PostMapping("/index")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.LIST_PERMISSION)
    public Response index(@RequestBody(required = false) Map<String,String> map) {

        return iTauvReportEvaluationService.getPageList(map);
    }

    /**
     * 编辑评价数据
     * @param reportEvaluation
     * @return
     */
    @PostMapping("/edit")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response edit(@RequestBody @Valid TauvReportEvaluation reportEvaluation, BindingResult bindingResult) {
        if(bindingResult.hasErrors()){
            String msg = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(msg);
        }
        return iTauvReportEvaluationService.edit(reportEvaluation);
    }

    /**
     * 获取评价数据
     * @param id
     * @return
     */
    @GetMapping("/getReportEvaluation")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response getReportEvaluation(Integer id) {
        if (id <= 0) {
            return response.failure("对应数据不存在");
        }
        TauvReportEvaluation info = iTauvReportEvaluationService.getInfoById(id);
        if (info == null) {
            return response.failure("对应数据不存在");
        }
        return response.success(info);
    }

    /**
     * 添加评价数据
     * @param reportEvaluation
     * @return
     */
    @PostMapping("/add")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.ADD_PERMISSION)
    public Response add(@RequestBody @Valid TauvReportEvaluation reportEvaluation, BindingResult bindingResult) {
        if(bindingResult.hasErrors()){
            String msg = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(msg);
        }
        return iTauvReportEvaluationService.add(reportEvaluation);
    }

    /**
     * 修改状态
     * @param map
     * @return
     */
    @PostMapping("/setStatus")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.STATUS_PERMISSION)
    public Response setStatus(@RequestBody Map<String,String> map) {
        return iTauvReportEvaluationService.setStatus(map);
    }
}
