package com.taauav.admin.entity;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.taauav.common.domain.Entity;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.NotNull;

/**
 * <p>
 * 系统配置表
 * </p>
 *
 * @author dyg
 * @since 2019-11-12
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
public class SysConfig extends Entity {

    private static final long serialVersionUID = 1L;

    /**
     * 变量名
     */
    @TableField("`name`")
    @NotNull(message = "配置名称不能为空")
    @Length(min = 2, max = 50, message = "配置名称长度为2-50个字")
    private String name;

    /**
     * 分组
     */
    @TableField("`group`")
    @NotNull(message = "请选择配置分组")
    private Integer group;

    /**
     * 变量标题
     */
    @NotNull(message = "配置标题不能为空")
    @Length(min = 2, max = 50, message = "配置标题长度为2-50个字")
    private String title;

    /**
     * 变量描述
     */
    @Length(max = 100, message = "配置描述不能超过100字")
    private String tip;

    /**
     * 类型：string,text,int,bool,array,datetime,date,file
     */
    @TableField("`type`")
    @NotNull(message = "请选择配置类型")
    private String type;

    /**
     * 输入类型
     */
    @NotNull(message = "请选择输入类型")
    private String inputType;

    /**
     * 变量值
     */
    @TableField("`value`")
    @NotNull(message = "配置值不能为空")
    private String value;

    /**
     * 变量字典数据
     */
    @NotNull(message = "字典数据不能为空")
    private String content;

    /**
     * 验证规则
     */
    @Length(max = 100, message = "验证规则不能超过100字")
    private String rule;

    /**
     * 扩展属性
     */
    @TableField("`extend`")
    private String extend;

    /**
     * 备注
     */
    @Length(max = 200, message = "备注不能超过200字")
    private String remark;

    /**
     * 排序
     */
    @TableField("`sort`")
    @NotNull(message = "请填写排序")
    private Integer sort;

    /**
     * 获取字段类型配置
     *
     * @return
     */
    public Map<String, String> getTypeList() {
        Map<String, String> typeList = new HashMap<>();
        typeList.put("string", "string");
        typeList.put("text", "text");
        typeList.put("int", "int");
        typeList.put("bool", "bool");
        typeList.put("array", "array");
        typeList.put("datetime", "datetime");
        typeList.put("date", "date");
        typeList.put("file", "file");
        return typeList;
    }

    /**
     * 获取输入类型配置
     *
     * @return
     */
    public Map<String, String> getInputTypeList() {
        Map<String, String> inputTypeList = new HashMap<>();
        inputTypeList.put("text", "文本");
        inputTypeList.put("textarea", "文本域");
        inputTypeList.put("switch", "开关");
        inputTypeList.put("radio", "单选按钮");
        inputTypeList.put("checkbox", "复选框");
        inputTypeList.put("select", "下拉框");
        return inputTypeList;
    }

    /**
     * 获取状态配置
     *
     * @return
     */
    public Map<Integer, String> getStatusList() {
        Map<Integer, String> statusList = new HashMap<>();
        statusList.put(1, "正常");
        statusList.put(2, "禁用");
        return statusList;
    }
}
