package com.taauav.admin.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.taauav.common.bean.Response;
import com.taauav.admin.dto.TauvReportDriverDTO;
import com.taauav.admin.entity.SysAdmin;
import com.taauav.admin.entity.TauvInspectFile;
import com.taauav.admin.entity.TauvReport;
import com.taauav.admin.mapper.SysAdminMapper;
import com.taauav.admin.service.IStatisticsService;
import com.taauav.admin.service.ISysCityService;
import com.taauav.admin.service.ITauvInspectFileService;
import com.taauav.admin.service.ITauvReportService;
import com.taauav.common.util.DateUtil;
import com.taauav.common.util.FunctionUtils;
import com.taauav.common.util.ShiroUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.*;

/**
 * <p>数据统计</p>
 *
 * @author : dyg
 * @date : 2019-12-05 17:08
 **/
@Service("iStatisticsService")
@Slf4j
public class StatisticsServiceImpl implements IStatisticsService {

    @Value("${server.UPLOAD_URL}")
    private String uploadUrl;
    @Autowired
    private ISysCityService iSysCityService;
    @Autowired
    private ITauvReportService iTauvReportService;
    @Autowired
    private ITauvInspectFileService iTauvInspectFileService;
    @Autowired
    private Response response;
    @Value("${spring.pageSize}")
    private Integer pageSize;
    @Autowired
    private SysAdminMapper adminMapper;

    /**
     * 获取报告统计数据
     * @param map
     * @return
     */
    @Override
    public Response getStatisticsDataIndex(Map<String, String> map) {
        Map<String,String> query = getQueryCond(map);
        Integer startTime = !FunctionUtils.isEmpty("startTime", query) ? FunctionUtils.dateToStamp(query.get("startTime")) : 0;
        Integer endTime = !FunctionUtils.isEmpty("endTime", query) ? FunctionUtils.dateToStamp(query.get("endTime")) : 0;
        List<Map<String,Object>> cityList = iSysCityService.getCityListByPidAndAdmin(320100);
        List<Map<String,Object>> dataList = new ArrayList<>();
        if (!StringUtils.isEmpty(cityList)) {
            for (Map<String,Object> mp : cityList) {
                String cityId = mp.get("id").toString();
                query.put("driverArea", cityId);
                /*河道数量*/
                Integer dirverNum = iTauvReportService.getNumByCond(query);
                IPage<TauvReport> pageDataList = iTauvReportService.getListByCond(query, "");
                List<TauvReport> reportList = pageDataList.getRecords();
                /*获取问题数*/
                List<Map<String,Object>> fileDocumentNums = iTauvInspectFileService.getNumByFileDocument(cityId,0, startTime, endTime);
                /*水面，岸线，排口，水质数量初始化*/
                Map<String,Integer> waterCheckData = getWaterCheckInfo(fileDocumentNums);
                Integer surfaceWater = waterCheckData.get("surfaceWater");
                Integer shoreLine = waterCheckData.get("shoreLine");
                Integer outfall = waterCheckData.get("outfall");
                Integer waterQuality = waterCheckData.get("waterQuality");
                Map<String,Object> dtMap = new HashMap<>(9);
                dtMap.put("driverAreaId", cityId);
                dtMap.put("driverAreaName",mp.get("name"));
                dtMap.put("driverNum", dirverNum);
                dtMap.put("inspectNum", pageDataList.getTotal());
                dtMap.put("totalNum", surfaceWater+shoreLine+outfall+waterQuality);
                dtMap.put("surfaceWater",surfaceWater);
                dtMap.put("shoreLine",shoreLine);
                dtMap.put("outfall",outfall);
                dtMap.put("waterQuality",waterQuality);
                dataList.add(dtMap);
            }
        }
        if(CollectionUtils.isNotEmpty(dataList)){
            Collections.sort(dataList,new Comparator<Map<String,Object>>() {
                @Override
                public int compare(Map<String,Object> o1, Map<String,Object> o2) {
                    Integer totalNum1 = (Integer) o1.get("totalNum");
                    Integer totalNum2 = (Integer) o2.get("totalNum");
                    return totalNum2 - totalNum1;
                }
            });
        }

        return response.success(dataList);
    }

    /**
     * 获取区属河流列表
     * @param map
     * @return
     */
    @Override
    public Response getAreaStaticsData(Map<String, String> map) {
        Map<String,String> query = getQueryCond(map);
        String cityId = query.get("driverArea");
//        String driverExam = query.get("driverExam");
        Integer startTime = !FunctionUtils.isEmpty("startTime", query) ? FunctionUtils.dateToStamp(query.get("startTime")) : 0;
        Integer endTime = !FunctionUtils.isEmpty("endTime", query) ? FunctionUtils.dateToStamp(query.get("endTime")) : 0;
        List<TauvReportDriverDTO> dataList = new ArrayList<>();
        /*河道数量：*/
        Integer driverNum = iTauvReportService.getNumByCond(query);
        /* 问题数量：*/
        List<Map<String,Object>> cityFileDocumentNums = iTauvInspectFileService.getNumByFileDocument(cityId,0, startTime, endTime);
        Map<String,Integer> cityWaterCheckData = getWaterCheckInfo(cityFileDocumentNums);
        Integer questionNum = cityWaterCheckData.get("totalNum");

        IPage<TauvReport> pageDataList = iTauvReportService.getListByCond(query, "inspect_no asc");
        List<TauvReport> reportList = pageDataList.getRecords();
        if (null != reportList) {
            for (TauvReport report : reportList) {
                TauvReportDriverDTO info = new TauvReportDriverDTO();
                info.setId(report.getId());
                info.setDriverName(report.getDriverName());
                info.setInspectTime(report.getInspectTime());
                if (!StringUtils.isEmpty(report.getInspectTime())) {
                    info.setFormatInspectTime(DateUtil.format(report.getInspectTime(),"yyyy-MM-dd HH:mm:ss"));
                }
                info.setTotalExamineScore(report.getTotalExamineScore());
                List<Map<String,Object>> fileDocumentNums = iTauvInspectFileService.getNumByFileDocument(cityId, report.getInspectDriverId(), startTime, endTime);
                /*水面，岸线，排口，水质数量初始化*/
                Map<String,Integer> waterCheckData = getWaterCheckInfo(fileDocumentNums);
                log.warn("checkData=" + JSON.toJSONString(waterCheckData));
                Integer surfaceWater = waterCheckData.get("surfaceWater");
                Integer shoreLine = waterCheckData.get("shoreLine");
                Integer outfall = waterCheckData.get("outfall");
                Integer waterQuality = waterCheckData.get("waterQuality");
                Integer totalNum = waterCheckData.get("totalNum");
                info.setTotalNum(totalNum);
                info.setSurfaceWater(surfaceWater);
                info.setShoreLine(shoreLine);
                info.setOutfall(outfall);
                info.setWaterQuality(waterQuality);
                dataList.add(info);
            }
            Comparator<TauvReportDriverDTO> byTotalNum = Comparator.comparing(TauvReportDriverDTO::getTotalNum).reversed();
            Comparator<TauvReportDriverDTO> byTimeDesc = Comparator.comparing(TauvReportDriverDTO::getInspectTime).reversed();
            dataList.sort(byTotalNum.thenComparing(byTimeDesc));
        }
        Map<String,Object> result = new HashMap<>(4);
        result.put("data", dataList);
        result.put("pages", pageDataList.getPages());
        result.put("current", pageDataList.getCurrent());
        result.put("total", pageDataList.getTotal());
        result.put("driverNum", driverNum);
        result.put("inspectNum", pageDataList.getTotal());
        result.put("questionNum", questionNum);
        result.put("size", pageDataList.getSize());
        return response.success(result);
    }

    /**
     * 根据报告编号获取问题信息
     * @param reportId
     * @return
     */
    @Override
    public Response getInfoByReportId(Integer reportId, String fileDocument) {
        if (StringUtils.isEmpty(reportId)) {
            return response.failure("编号不存在");
        }
        TauvReport reportInfo = iTauvReportService.getInfoById(reportId);
        if (StringUtils.isEmpty(reportInfo)) {
            return response.failure("数据不存在");
        }
        Map<String,Object> result = new HashMap<>(8);
        result.put("id",reportId);
        result.put("reportNo", reportInfo.getReportNo());
        result.put("driverName", reportInfo.getDriverName());
        result.put("driverArea", reportInfo.getDriverAreaText());
        result.put("formatInspectTime", reportInfo.getFormatInspectTime());
        String questionType = "";
        Integer questionNum = 0;
        TauvInspectFile inspectFile = new TauvInspectFile();
        Map<String,String> documentList = inspectFile.getFileDocumentList();
        if (!documentList.containsKey(fileDocument)) {
            fileDocument = "";
        } else {
            questionType = documentList.get(fileDocument);
        }
        /*问题数据*/
        List<Map<String,Object>> fileList = iTauvInspectFileService.getReviewDataByInspectDriverIdAndFileDocument(reportInfo.getInspectDriverId(),fileDocument);
        if (StringUtils.isEmpty(fileList)) {
            for (Map<String,Object> mp : fileList) {
                String originalImg = mp.getOrDefault("original_img", "").toString();
                String src = mp.getOrDefault("src", "").toString();
                if (!StringUtils.isEmpty(originalImg)) {
                    mp.put("formatOriginalImg",uploadUrl + originalImg);
                }
                if (!StringUtils.isEmpty(src)) {
                    mp.put("formatSrc", uploadUrl + src);
                }
            }
        }
        questionNum = fileList.size();
        result.put("questionList", fileList);
        result.put("questionType", questionType);
        result.put("questionNum", questionNum);
        return response.success(result);
    }

    /**
     * 获取搜索条件数据
     * @param map
     * @return
     */
    private Map<String,String> getQueryCond(Map<String, String> map) {
        map = StringUtils.isEmpty(map) ? new HashMap<>() : map;
        Map<String,String> query = new HashMap<>(5);
        String driverArea = "0";
        if (!FunctionUtils.isEmpty("driverArea", map)) {
            driverArea = map.getOrDefault("driverArea", "0");
        }
        String inspectTime = "";
        if (!FunctionUtils.isEmpty("inspectTime", map)) {
            inspectTime = map.getOrDefault("inspectTime", "");
        }
        String type = "1";
        if (!FunctionUtils.isEmpty("type", map)) {
            type = map.getOrDefault("type", "1");
        }
        Integer pgSize = pageSize;
        if (!FunctionUtils.isEmpty("pageSize", map) && Integer.valueOf(map.get("pageSize")) > 0) {
            pgSize = Integer.valueOf(map.get("pageSize"));
        }
        Integer pageNo = 1;
        if (!FunctionUtils.isEmpty("page", map) && Integer.valueOf(map.get("page")) > 0) {
            pageNo = Integer.valueOf(map.get("page"));
        }
        String startTime = "";
        String endTime = "";
        if (!StringUtils.isEmpty(inspectTime)) {
            String[] inspectTimeArr = inspectTime.split("~");
            startTime = StringUtils.isEmpty(inspectTimeArr[0]) ? "0" : inspectTimeArr[0] + " 00:00:00";
            endTime = StringUtils.isEmpty(inspectTimeArr[1]) ? "0" : inspectTimeArr[1] + " 23:59:59";
        }
        if (StringUtils.isEmpty(driverArea) || driverArea.equals("0")) {
            Integer adminId = ShiroUtils.getAdminId();
            SysAdmin adminInfo = adminMapper.selectById(adminId);
            driverArea = adminInfo.getCityIds();
        }
//        query.put("driverExam", type);
        query.put("startTime", startTime);
        query.put("endTime", endTime);
        query.put("driverArea", driverArea);
        query.put("pageSize", pgSize.toString());
        query.put("page", pageNo.toString());
        return query;
    }

    /**
     * 获取水质检测数据
     * @param fileDocumentNums
     * @return
     */
    private Map<String,Integer> getWaterCheckInfo(List<Map<String,Object>> fileDocumentNums) {
        /*水面，岸线，排口，水质数量初始化*/
        Integer surfaceWater = 0;
        Integer shoreLine = 0;
        Integer outfall = 0;
        Integer waterQuality = 0;
        Integer total = 0;
        Map<String,Integer> info = new HashMap<>(5);
        if (null != fileDocumentNums && fileDocumentNums.size() > 0) {
            for(Map<String,Object> docMap : fileDocumentNums) {
                String fileDocument = docMap.get("file_document").toString();
                Long totalNumL = (Long) docMap.get("total_num");
                Integer totalNum = totalNumL.intValue();
                if (fileDocument.equals("surfaceWater")) {
                    surfaceWater = totalNum;
                } else if (fileDocument.equals("shoreLine")) {
                    shoreLine = totalNum;
                } else if (fileDocument.equals("outfall")) {
                    outfall = totalNum;
                } else if(fileDocument.equals("waterQuality")) {
                    waterQuality = totalNum;
                }
                total += totalNum;
            }
        }
        info.put("totalNum", total);
        info.put("surfaceWater",surfaceWater);
        info.put("shoreLine",shoreLine);
        info.put("outfall",outfall);
        info.put("waterQuality",waterQuality);
        return info;
    }
}
