package com.taauav.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.admin.entity.LsAuthGroup;
import com.taauav.common.bean.CacheUser;
import com.taauav.common.bean.Response;
import com.taauav.common.constant.SysAdminConstant;
import com.taauav.admin.entity.SysAdmin;
import com.taauav.admin.entity.SysAuthGroup;
import com.taauav.admin.mapper.SysAdminMapper;
import com.taauav.admin.mapper.SysAuthGroupAccessMapper;
import com.taauav.admin.mapper.SysAuthGroupMapper;
import com.taauav.admin.service.ISysAdminService;
import com.taauav.admin.service.ISysAuthGroupAccessService;
import com.taauav.admin.service.ISysAuthRuleService;
import com.taauav.admin.service.ISysCityService;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.DateUtil;
import com.taauav.common.util.FunctionUtils;
import com.taauav.common.util.ShiroUtils;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authc.AuthenticationException;
import org.apache.shiro.authc.IncorrectCredentialsException;
import org.apache.shiro.authc.UnknownAccountException;
import org.apache.shiro.authc.UsernamePasswordToken;
import org.apache.shiro.subject.Subject;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import javax.validation.Validator;
import java.time.LocalDateTime;
import java.util.*;


/**
 * <p>
 * 管理员表 服务实现类
 * </p>
 *
 * @author dyg
 * @since 2019-11-04
 */
@Service
public class SysAdminServiceImpl extends BaseServiceImpl<SysAdminMapper, SysAdmin> implements ISysAdminService {
    @Resource
    private Response response;
    @Value("${spring.pageSize}")
    private Integer pageSize;
    @Autowired
    private SysAdminMapper sysAdminMapper;
    @Autowired
    private SysAuthGroupMapper authGroupMapper;
    @Resource
    private Validator validator;
    @Autowired
    private SysAuthGroupServiceImpl authGroupService;
    @Autowired
    private ISysAuthGroupAccessService iSysAuthGroupAccessService;
    @Autowired
    private ISysAuthRuleService iSysAuthRuleService;
    @Autowired
    private SysAuthGroupAccessMapper accessMapper;
    @Autowired
    private ISysCityService cityService;
    @Value("${server.IMAGE_URL}")
    private String imageUrl;

    /**
     * 根据用户名获取用户
     *
     * @param username
     * @return
     */
    @Override
    public SysAdmin findByUsername(String username) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("username", username);
        wrapper.eq("mark", 1);
        SysAdmin admin = sysAdminMapper.selectOne(wrapper);
        if (admin != null && !"".equals(admin.getAvatar())) {
            admin.setAvatar(imageUrl + admin.getAvatar());
        }
        return admin;
//        baseMapper.selectOne(
//                new LambdaQueryWrapper<SysAdmin>().eq(SysAdmin::getUsername, username)
//        );
    }

    /**
     * 根据用户编号获取数据
     *
     * @param id
     * @return
     */
    @Override
    public SysAdmin findById(Integer id) {
        return baseMapper.selectOne(
                new LambdaQueryWrapper<SysAdmin>().eq(SysAdmin::getId, id)
        );
    }

    /**
     * 用户登陆
     *
     * @param username
     * @param password
     * @return
     */
    @Override
    public Response login(String username, String password) {
        // 获取Subject实例对象，用户实例
        Subject currentUser = SecurityUtils.getSubject();

        // 将用户名和密码封装到UsernamePasswordToken
        UsernamePasswordToken token = new UsernamePasswordToken(username, password);

        CacheUser cacheUser;

        // 4、认证
        try {
            // 传到 MyShiroRealm 类中的方法进行认证
            currentUser.login(token);
            // 构建缓存用户信息返回给前端
            SysAdmin user = (SysAdmin) currentUser.getPrincipals().getPrimaryPrincipal();
            cacheUser = CacheUser.builder()
                    .token(currentUser.getSession().getId().toString())
                    .build();
            BeanUtils.copyProperties(user, cacheUser);
        } catch (UnknownAccountException e) {
            log.error("账户不存在异常：", e);
            return response.failure("账号不存在!");
//            throw new LoginException("账号不存在!", e);
        } catch (IncorrectCredentialsException e) {
            log.error("凭据错误（密码错误）异常：", e);
            return response.failure("密码不正确!");
//            throw new LoginException("密码不正确!", e);
        } catch (AuthenticationException e) {
            log.error("身份验证异常:", e);
            return response.failure("用户验证失败!");
//            throw new LoginException("用户验证失败!", e);
        }
        Map<String, String> result = new HashMap<>();
        result.put("token", cacheUser.getToken());
        return response.success("", result);
    }

    /**
     * 用户退出
     */
    @Override
    public void logout() {
        Subject subject = SecurityUtils.getSubject();
        subject.logout();
    }

    /**
     * 获取所有用户数据
     *
     * @param map
     * @return
     */
    @Override
    public Response findAdminPageList(Map<String, String> map) {
        QueryWrapper<SysAdmin> wrapper = new QueryWrapper<>();
        wrapper.eq("mark", 1);
        wrapper.orderByAsc("status");
        wrapper.orderByAsc("id");
        Integer pageNo = 1;
        if (map != null) {
            //页数
            String pageStr = "page";
            if (map.containsKey(pageStr) && !"".equals(map.get(pageStr)) && Integer.valueOf(map.get(pageStr)) > 0) {
                pageNo = Integer.valueOf(map.get(pageStr));
            }
            //关键词筛选条件
            String keywords = map.containsKey("keywords") ? map.get("keywords") : "";
            if (keywords != "" && !keywords.equals(null)) {
                wrapper.like("username", keywords).or().like("realname", keywords);
            }
            //公司名称筛选条件
            String company = map.getOrDefault("company", "");
            if (company != "" && !company.equals(null)) {
                wrapper.like("company", company);
            }
            //角色筛选条件
            String authGroup = map.getOrDefault("authGroup", "");
            if (!StringUtils.isEmpty(authGroup)) {
                if (FunctionUtils.isNumeric(authGroup)) {
                    wrapper.eq("auth_group", authGroup);
                } else {
                    return response.failure("对应角色不存在！");
                }
            }
            //每页展示数量
            String pgSize = "pageSize";
            if (map.containsKey(pgSize) && Integer.valueOf(map.get(pgSize)) > 0) {
                pageSize = Integer.valueOf(map.get(pgSize));
            }
        }
        double totalNum = sysAdminMapper.selectCount(wrapper);
        Integer totalPages = (int) (Math.ceil(totalNum / pageSize));
        if (pageNo > totalPages) {
            pageNo = totalPages;
        }
        IPage<SysAdmin> page = new Page<>(pageNo, pageSize);
        IPage data = this.page(page, wrapper);
        Map<String, Object> list = new HashMap<>(5);
        if (data.getRecords() != null) {
            list.put("total", data.getTotal());
            list.put("size", data.getSize());
            list.put("current", data.getCurrent());
            list.put("pages", data.getPages());
            List<SysAdmin> adminList = data.getRecords();
            List<Object> records = new ArrayList<>();
            adminList.forEach(val -> {
                Map<String, Object> mp = new HashMap<>(6);
                mp.put("id", val.getId());
                mp.put("realname", val.getRealname());
                mp.put("username", val.getUsername());
                mp.put("company", val.getCompany());
                mp.put("status", val.getStatus());
                mp.put("duty", val.getDuty());
                mp.put("authGroup", val.getAuthGroup());
                mp.put("mobile", val.getMobile());
                mp.put("cityIds", val.getCityIds());
                mp.put("rules", val.getRules());
                //处理角色名称
                String authGroupText = "";
                if (!StringUtils.isEmpty(val.getAuthGroup())) {
                    authGroupText = authGroupService.getAuthText(Integer.valueOf(val.getAuthGroup()));
                }
                mp.put("authGroupText", authGroupText);
                //处理状态名称
                String statusText = FunctionUtils.getArrayText(String.valueOf(val.getStatus()), SysAdminConstant.getAdminStatus());
                mp.put("statusText", statusText);
                records.add(mp);
            });
            list.put("records", records);
        }
        return response.success(list);
    }

    /**
     * 获取管理员数据信息
     *
     * @param id
     * @return
     */
    @Override
    public SysAdmin getAdminInfo(Integer id) {
        if (id <= 0) {
            return null;
        }
        SysAdmin admin = sysAdminMapper.selectById(id);
        if (admin == null) {
            return null;
        }
        //处理头像
        if (!"".equals(admin.getAvatar())) {
            admin.setAvatar(imageUrl + admin.getAvatar());
        }
        //处理角色名称
        String authText = authGroupService.getAuthText(Integer.valueOf(admin.getAuthGroup()));
        admin.setAuthGroupText(authText);
        //处理状态名称
        String statusText = FunctionUtils.getArrayText(String.valueOf(admin.getStatus()), SysAdminConstant.getAdminStatus());
        admin.setStatusText(statusText);
        return admin;
    }


    /**
     * 保存管理员数据
     *
     * @param map
     * @return
     */
    @Override
    public Response editAdminInfo(SysAdmin map) {
        if (map == null) {
            return response.failure("参数不能为空");
        }
        //数据组装
        SysAdmin admin = new SysAdmin();
        Integer id = map.getId();
        if (id != null) {
            admin.setId(id);
        } else {
            admin.setPassword(FunctionUtils.password("123456"));
        }
        String realname = map.getRealname();
        admin.setRealname(realname);
        //用户名重复校验
        String username = map.getUsername();
        if (username != null) {
            QueryWrapper<SysAdmin> wrapper = new QueryWrapper<>();
            wrapper.eq("username", username);
            wrapper.eq("mark", 1);
            SysAdmin user = sysAdminMapper.selectOne(wrapper);
            if ((id == null && user != null) || (id != null && user != null && !id.equals(user.getId()))) {
                return response.failure("用户名已存在");
            }
            admin.setUsername(username);
        }
        admin.setCompany(map.getCompany());
        admin.setAuthGroup(map.getAuthGroup());
        //手机号重复校验
        String mobile = map.getMobile();
        if (!StringUtils.isEmpty(mobile)) {
            QueryWrapper wrp = new QueryWrapper();
            wrp.eq("mark", 1);
            wrp.eq("mobile", mobile);
            SysAdmin adminInfo = sysAdminMapper.selectOne(wrp);
            if ((id == null && adminInfo != null) || (id != null && adminInfo != null && !id.equals(adminInfo.getId()))) {
                return response.failure("手机号已存在");
            }
        }
        admin.setMobile(mobile);
        admin.setDuty(map.getDuty());
        String cityIds = map.getCityIds();
        if (cityIds != null) {
            admin.setCityIds(map.getCityIds());
        }
        if (id != null && id > 0) {
            admin.setUpdateUser(ShiroUtils.getAdminId());
            admin.setUpdateTime(DateUtil.now());
            Integer res = sysAdminMapper.updateById(admin);
        } else {
            admin.setCreateUser(ShiroUtils.getAdminId());
            admin.setCreateTime(DateUtil.now());
            Integer res = sysAdminMapper.insert(admin);
            id = admin.getId();
        }
        //设置权限
        if (!StringUtils.isEmpty(map.getAuthGroup())) {
            Map<String, String> groupMap = new HashMap<>();
            groupMap.put("adminId", id.toString());
            groupMap.put("groupIds", map.getAuthGroup());
            Map<String, String> result = iSysAuthGroupAccessService.add(id, map.getAuthGroup());
            if (!("200".equals(result.get("code")))) {
                return response.failure(result.get("msg"));
            }
        }
        return response.success("操作成功");
    }

    /**
     * 处理管理员mark标记、密码或状态
     *
     * @param map
     * @return
     */
    @Override
    public Response dealAdmin(Map<String, String> map) {
        String act = map.get("act");
        String id = map.getOrDefault("id", "0");
        if (Integer.valueOf(id) == 0) {
            return response.failure("管理员id不能为空");
        }
        SysAdmin adminInfo = sysAdminMapper.selectById(Integer.valueOf(id));
        if (adminInfo == null) {
            return response.failure("管理员信息不存在");
        }
        SysAdmin admin = new SysAdmin();
        admin.setId(Integer.valueOf(id));

        String drop = "drop";
        String reset = "reset";
        String changeSt = "changeStatus";
        String savePermit = "savePermit";
        if (drop.equals(act)) {
            //逻辑删除
            admin.setMark(0);
        } else if (reset.equals(act)) {
            //重置密码
            admin.setPassword(FunctionUtils.password("123456"));
        } else if (changeSt.equals(act)) {
            //修改状态
            String status = map.getOrDefault("status", "0");
            if (Integer.valueOf(status) == 0) {
                return response.failure("管理员状态不能为空");
            }
            if (Integer.valueOf(status) == 1) {
                admin.setStatus(Integer.valueOf("2"));
            } else {
                admin.setStatus(Integer.valueOf("1"));
            }
        } else if (savePermit.equals(act)) {
            //保存权限数据
            String rules = map.getOrDefault("rules", "");
            admin.setRules(rules);
        } else {
            return response.failure("操作方法不存在");
        }
        admin.setUpdateUser(ShiroUtils.getAdminId());
        admin.setUpdateTime(DateUtil.now());
        boolean result = updateById(admin);
        if (!result) {
            return response.failure("操作失败");
        }
        return response.success("操作成功");
    }

    /**
     * 获取权限数据
     *
     * @param map
     * @return
     */
    @Override
    public Response getPermit(Map<String, String> map) {
        String id = map.getOrDefault("id", "0");
        if (Integer.valueOf(id) == 0) {
            return response.failure("管理员id不存在");
        }
        List<Map<String, Object>> list = iSysAuthRuleService.getRuleList(Integer.valueOf(id));
        return response.success(list);
    }

    /**
     * 设置个人权限
     *
     * @param map
     * @return
     */
    @Override
    public Response setPermit(Map<String, String> map) {
        if (StringUtils.isEmpty(map)) {
            return response.failure("参数不全");
        }
        String adminIdStr = map.getOrDefault("id", "");
        if (StringUtils.isEmpty(adminIdStr)) {
            return response.failure("用户编号不存在");
        }
        Integer adminId = Integer.valueOf(adminIdStr);
        String rules = map.getOrDefault("rules", "");
//        if (StringUtils.isEmpty(rules)) {
//            return response.failure("请选择权限菜单");
//        }
        SysAdmin sysAdmin = new SysAdmin();
        sysAdmin.setId(adminId);
        sysAdmin.setRules(rules);
        sysAdminMapper.updateById(sysAdmin);
        //重置对应用户的权限
        SysAdmin adminInfo = getAdminInfo(adminId);
        String username = adminInfo.getUsername();
        ShiroUtils.deleteCache(username, false);
        return response.success("设置成功");
    }

    /**
     * 根据id获取管理员姓名
     *
     * @param id
     * @return
     */
    @Override
    public String getRealNameById(Integer id) {
        if (id <= 0) {
            return "";
        }
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("id", id);
        wrapper.eq("mark", 1);
        SysAdmin admin = getById(id);
        if (admin == null) {
            return "";
        }
        return admin.getRealname();
    }

    /**
     * 根据角色获取用户
     *
     * @param groupId
     * @return
     */
    @Override
    public List<Map<String, Object>> getAdminListByGroupId(Integer groupId) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("mark", 1);
        //wrapper.
        List<Map<String, Object>> list = sysAdminMapper.selectListByGroupId(groupId);
        return list;
    }

    /**
     * 获取管理员一维数组
     *
     * @return
     */
    @Override
    public Response getAdminList() {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("mark", 1);
        List<SysAdmin> list = sysAdminMapper.selectList(wrapper);
        return response.success("操作成功", list);
    }

    /**
     * 获取角色数组
     *
     * @return
     */
    @Override
    public Response getAuthGroupList() {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("mark", 1);
        wrapper.gt("pid", 0);
        List<SysAuthGroup> authList = authGroupMapper.selectList(wrapper);
        List<Object> list = new ArrayList<>();
        if (authList != null) {
            authList.forEach(val -> {
                Map<String, Object> map = new HashMap<>();
                map.put("id", val.getId());
                SysAuthGroup pInfo = authGroupMapper.selectById(val.getPid());
                String pName;
                if (pInfo != null) {
                    pName = pInfo.getName() + "-" + val.getName();
                } else {
                    pName = val.getName();
                }
                map.put("name", pName);
                list.add(map);
            });
        }
        return response.success("操作成功", list);
    }

    /**
     * 根据ID获取人员名称
     *
     * @param adminIds 人员ID(多个逗号分隔)
     * @return
     */
    @Override
    public List<String> getAdminNameByIds(String adminIds) {
        // 查询条件
        QueryWrapper<SysAdmin> queryWrapper = new QueryWrapper<>();
        // 待分配任务
        String arr[] = adminIds.split(",");
        ArrayList arrayList = new ArrayList();
        for (String s : arr) {
            arrayList.add(s);
        }
        queryWrapper.in("id", arrayList);
        queryWrapper.eq("mark", 1);

        // 查询数据
        List<SysAdmin> adminList = sysAdminMapper.selectList(queryWrapper);
        List<String> stringList = new ArrayList<>();
        if (!adminList.isEmpty()) {
            adminList.forEach(item -> {
                stringList.add(item.getRealname());
            });
        }
        return stringList;
    }
}
