package com.taauav.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.common.bean.Response;
import com.taauav.common.constant.SysLevelConstant;
import com.taauav.admin.entity.SysAdmin;
import com.taauav.admin.entity.SysLevel;
import com.taauav.admin.mapper.SysAdminMapper;
import com.taauav.admin.mapper.SysLevelMapper;
import com.taauav.admin.service.ISysLevelService;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.DateUtil;
import com.taauav.common.util.FunctionUtils;
import com.taauav.common.util.ShiroUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.validation.ConstraintViolation;
import javax.validation.Validator;
import java.util.*;

/**
 * <p>
 * 职级管理表 服务实现类
 * </p>
 *
 * @author dyg
 * @since 2019-11-11
 */
@Service
public class SysLevelServiceImpl extends BaseServiceImpl<SysLevelMapper, SysLevel> implements ISysLevelService {
    @Autowired
    private SysLevelMapper levelMapper;
    @Autowired
    private Response response;
    @Value("${spring.pageSize}")
    private Integer pageSize;
    @Autowired
    private SysAdminMapper adminMapper;
    @Resource
    private Validator validator;

    /**
     * 获取列表数据
     *
     * @param map
     * @return
     */
    @Override
    public Response pageList(Map<String, String> map) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("mark", 1);
        if (map != null) {
            String keywords = "keywords";
            if (map.containsKey(keywords)) {
                if (FunctionUtils.isNumeric(map.get(keywords))) {
                    wrapper.eq("id", Integer.valueOf(map.get(keywords)));
                } else if (!"".equals(map.get(keywords))) {
                    wrapper.like("name", map.get(keywords));
                }
            }
            String status = "status";
            if (map.containsKey(status) && FunctionUtils.isNumeric(map.get(status))) {
                wrapper.eq("status", Integer.valueOf(map.get(status)));
            }
            String createUser = "createUser";
            if (map.containsKey(createUser) && FunctionUtils.isNumeric(map.get(createUser))) {
                wrapper.eq("create_user", Integer.valueOf(map.get(createUser)));
            }
            //每页数量
            String pgSize = "pageSize";
            if (map.containsKey(pgSize) && Integer.valueOf(map.get(pgSize)) > 0) {
                pageSize = Integer.valueOf(map.get(pgSize));
            }
        }

        Integer pageNo = map.containsKey("page") && Integer.valueOf(map.get("page")) > 0 ? Integer.valueOf(map.get("page")) : 1;
        double total = levelMapper.selectCount(wrapper);
        Integer totalPage = (int) Math.ceil(total / pageSize);
        if (pageNo > totalPage) {
            pageNo = totalPage;
        }
        IPage<SysLevel> page = new Page<>(pageNo, pageSize);
        IPage data = this.page(page, wrapper);
        Map<String, Object> list = new HashMap<>(6);
        list.put("total", data.getTotal());
        list.put("size", data.getSize());
        list.put("current", data.getCurrent());
        list.put("pages", data.getPages());
        if (data.getRecords() != null) {
            List<SysLevel> levelList = data.getRecords();
            List<Object> records = new ArrayList<>();
            levelList.forEach(val -> {
                Map<String, Object> mp = new HashMap<>(10);
                mp.put("id", val.getId());
                mp.put("name", val.getName());
                mp.put("remark", val.getRemark());
                mp.put("sort", val.getSort());
                mp.put("status", val.getStatus());
                mp.put("statusText", FunctionUtils.getArrayText(String.valueOf(val.getStatus()), SysLevelConstant.getLevelStatus()));
                mp.put("createUser", val.getCreateUser());
                SysAdmin admin = adminMapper.selectById(val.getCreateUser());
                if (admin != null) {
                    mp.put("createUsername", admin.getRealname());
                } else {
                    mp.put("createUsername", "");
                }
                mp.put("createTime", val.getCreateTime());
                mp.put("formatCreateTime", val.getCreateTime());
                records.add(mp);
            });
            list.put("records", records);
        }
        return response.success("操作成功", list);
    }

    /**
     * 添加、编辑
     *
     * @param map
     * @return
     */
    @Override
    public Response edit(SysLevel map) {
        //必填字段及格式校验
        Set<ConstraintViolation<SysLevel>> violations = validator.validate(map);
        for (ConstraintViolation<SysLevel> violation : violations) {
            if (!"".equals(violation.getMessage())) {
                return response.failure(violation.getMessage());
            }
        }
        SysLevel level = new SysLevel();
        level.setName(map.getName());
        String remark = map.getRemark();
        if (remark != null) {
            level.setRemark(remark);
        }
        level.setStatus(map.getStatus());
        level.setSort(map.getSort());
        Integer id = map.getId();
        boolean res = true;
        if (id != null && id > 0) {
            level.setId(id);
            res = this.editData(level);
        } else {
            res = this.addData(level);
        }
        if (!res) {
            return response.failure("操作失败");
        }
        return response.success("操作成功");
    }

    /**
     * 根据id获取对应职级数据
     *
     * @param id
     * @return
     */
    @Override
    public Response getLevelInfo(Integer id) {
        if (id <= 0) {
            return response.failure("职级id必须大于0");
        }
        SysLevel levelInfo = levelMapper.selectById(id);
        if (levelInfo == null) {
            return response.failure("对应职级数据不存在");
        }
        Map<String, Object> info = new HashMap<>(14);
        info.put("id", levelInfo.getId());
        info.put("name", levelInfo.getName());
        info.put("status", levelInfo.getStatus());
        info.put("statusText", FunctionUtils.getArrayText(String.valueOf(levelInfo.getStatus()), SysLevelConstant.getLevelStatus()));
        info.put("remark", levelInfo.getRemark());
        info.put("sort", levelInfo.getSort());
        info.put("createUser", levelInfo.getCreateUser());
        String createUserName = "";
        if (levelInfo.getCreateUser() > 0) {
            SysAdmin admin = adminMapper.selectById(levelInfo.getCreateUser());
            if (admin != null) {
                createUserName = admin.getRealname();
            }
        }
        info.put("createUserName", createUserName);
        info.put("createTime", levelInfo.getCreateTime());
        String formatCreateTime = "";
        if (levelInfo.getCreateTime() != null) {
            formatCreateTime = levelInfo.getCreateTime().toString();
        }
        info.put("formatCreateTime", formatCreateTime);
        info.put("updateUser", levelInfo.getUpdateUser());
        String updateUserName = "";
        if (levelInfo.getUpdateUser() > 0) {
            SysAdmin admin = adminMapper.selectById(levelInfo.getUpdateUser());
            if (admin != null) {
                updateUserName = admin.getRealname();
            }
        }
        info.put("updateUserName", updateUserName);
        info.put("updateTime", levelInfo.getUpdateTime());
        String formatUpdateTime = "";
        if (levelInfo.getUpdateTime() != null) {
            formatUpdateTime = levelInfo.getUpdateTime().toString();
        }
        info.put("formatUpdateTime", formatUpdateTime);
        return response.success("操作成功", info);
    }

    /**
     * 根据id删除职级
     *
     * @param id
     * @return
     */
    @Override
    public Response drop(Integer id) {
        if (id <= 0) {
            return response.failure("职级id必须大于0");
        }
        SysLevel level = levelMapper.selectById(id);
        if (level == null) {
            return response.failure("职级数据不存在");
        }
        level.setUpdateUser(ShiroUtils.getAdminId());
        level.setUpdateTime(DateUtil.now());
        level.setMark(0);
        Boolean res = updateById(level);
        if (res.equals(false)) {
            return response.failure("操作失败");
        }
        return response.success("操作成功");
    }

    /**
     * 修改职级状态
     *
     * @param id
     * @param status
     * @return
     */
    @Override
    public Response changeStatus(Integer id, Integer status) {
        if (id < 1) {
            return response.failure("职级id必须大于0");
        }
        SysLevel info = levelMapper.selectById(id);
        if (info == null) {
            return response.failure("职级数据不存在");
        }
        Integer st = 0;
        if (status == 1) {
            st = 2;
        } else {
            st = 1;
        }
        info.setStatus(st);
        info.setUpdateUser(ShiroUtils.getAdminId());
        info.setUpdateTime(DateUtil.now());
        Boolean res = updateById(info);
        if (res.equals(false)) {
            return response.failure("操作失败");
        }
        return response.success("操作成功");
    }
}
