package com.taauav.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.admin.constant.CommonConstant;
import com.taauav.admin.entity.TauvMountEquipment;
import com.taauav.admin.mapper.TauvMountEquipmentMapper;
import com.taauav.admin.query.MountEquipmentQuery;
import com.taauav.admin.service.ITauvMountEquipmentService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.taauav.admin.vo.TauvMountEquipmentListVo;
import com.taauav.common.bean.Response;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.FunctionUtils;
import com.taauav.common.util.ShiroUtils;
import com.taauav.common.util.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 挂载设备表 服务实现类
 * </p>
 *
 * @author dyg
 * @since 2020-04-01
 */
@Service
public class TauvMountEquipmentServiceImpl extends BaseServiceImpl<TauvMountEquipmentMapper, TauvMountEquipment> implements ITauvMountEquipmentService {

    @Autowired
    private TauvMountEquipmentMapper mountEquipmentMapper;

    @Autowired
    private Response response;

    /**
     * @param query 查询条件
     * @return
     */
    @Override
    public Response getList(MountEquipmentQuery query) {
        // 查询条件
        QueryWrapper<TauvMountEquipment> queryWrapper = new QueryWrapper<>();
        // 设备名称
        if (!StringUtils.isEmpty(query.getName())) {
            queryWrapper.like("name", query.getName());
        }
        // 设备编号
        if (!StringUtils.isEmpty(query.getEquipmentSn())) {
            queryWrapper.like("equipment_sn", query.getEquipmentSn());
        }
        // 设备类型
        if (query.getType() != null && query.getType() > 0) {
            queryWrapper.eq("type", query.getType());
        }
        queryWrapper.eq("mark", 1);
        queryWrapper.orderByDesc("id");

        // 查询数据
        IPage<TauvMountEquipment> page = new Page<>(query.getPage(), query.getPageSize());
        IPage<TauvMountEquipment> data = mountEquipmentMapper.selectPage(page, queryWrapper);
        List<TauvMountEquipment> tauvMountEquipmentList = data.getRecords();
        List<TauvMountEquipmentListVo> tauvMountEquipmentListVoList = new ArrayList<>();
        if (!tauvMountEquipmentList.isEmpty()) {
            tauvMountEquipmentList.forEach(item -> {
                TauvMountEquipmentListVo tauvMountEquipmentListVo = new TauvMountEquipmentListVo();
                // 拷贝属性
                BeanUtils.copyProperties(item, tauvMountEquipmentListVo);
                // 设备类型
                tauvMountEquipmentListVo.setTypeName(CommonConstant.MOUNT_EQUIPMENT_TYPE_LIST.get(item.getType()));
                // 设备状态
                tauvMountEquipmentListVo.setStatusName(CommonConstant.MOUNT_EQUIPMENT_TYPE_LIST.get(item.getStatus()));
                tauvMountEquipmentListVoList.add(tauvMountEquipmentListVo);
            });
        }
        // 返回结果
        Map<String, Object> result = new HashMap<>();
        result.put("total", data.getTotal());
        result.put("size", data.getSize());
        result.put("current", data.getCurrent());
        result.put("pages", data.getPages());
        result.put("records", tauvMountEquipmentListVoList);
        return response.success(result);
    }

    /**
     * 添加挂载设备
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response add(TauvMountEquipment entity) {
        if (entity == null) {
            return response.failure("实体对象不能为空");
        }
        boolean result = this.addData(entity);
        if (!result) {
            return response.failure("添加失败");
        }
        // 设置设备编码
        String equipmentSn = String.format("%05d", entity.getId());
        entity.setEquipmentSn(equipmentSn);
        int rs = mountEquipmentMapper.updateById(entity);
        if (rs == 0) {
            return response.failure("设备编码更新失败");
        }
        return response.success("添加成功");
    }

    /**
     * 获取挂载设备详情
     *
     * @param id 挂载设备ID
     * @return
     */
    @Override
    public Response info(Integer id) {
        TauvMountEquipment entity = this.getInfo(id);
        if (entity == null) {
            return response.failure("数据存在");
        }
        return response.success(entity);
    }

    /**
     * 编辑挂载设备
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response edit(TauvMountEquipment entity) {
        if (entity == null || entity.getId() == null) {
            return response.failure("实体对象不能为空");
        }
        boolean result = this.editData(entity);
        if (!result) {
            return response.failure("编辑失败");
        }
        return response.success("编辑成功");
    }

    /**
     * 删除挂载设备
     *
     * @param id 设备ID
     * @return
     */
    @Override
    public Response delete(Integer id) {
        if (id == null || id <= 0) {
            return response.failure("设备ID不存在");
        }
        TauvMountEquipment entity = mountEquipmentMapper.selectById(id);
        if (entity == null) {
            return response.failure("设备信息不存在");
        }
        boolean result = this.deleteById(id, entity);
        if (!result) {
            return response.failure("删除失败");
        }
        return response.success("删除成功");
    }

    /**
     * 设置状态
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response setStatus(TauvMountEquipment entity) {
        if (entity.getId() == null || entity.getId() <= 0) {
            return response.failure("设备ID不存在");
        }
        if (entity.getStatus() == null || entity.getStatus() <= 0) {
            return response.failure("状态不存在");
        }
        int result = mountEquipmentMapper.updateById(entity);
        if (result == 0) {
            return response.failure("设置失败");
        }
        return response.success("设置成功");
    }

    /**
     * 获取参数信息
     *
     * @return
     */
    @Override
    public Response getParamsList() {
        Map<String, Object> result = new HashMap<>();
        result.put("type_list", CommonConstant.MOUNT_EQUIPMENT_TYPE_LIST);
        return response.success(result);
    }

    /**
     * 获取挂载设备列表
     *
     * @param query 查询条件
     * @return
     */
    @Override
    public Response getMountEquipmentList(MountEquipmentQuery query) {
        // 查询条件
        QueryWrapper<TauvMountEquipment> queryWrapper = new QueryWrapper<>();
        // 设备类型
        if (query.getType() != null && query.getType() > 0) {
            queryWrapper.eq("type", 1);
        }
        // 设备型号
        if (!StringUtils.isEmpty(query.getModel())) {
            queryWrapper.like("model", query.getModel());
        }
        queryWrapper.eq("status", 1);
        queryWrapper.eq("mark", 1);
        queryWrapper.orderByDesc("id");

        // 查询数据
        List<TauvMountEquipment> mountEquipmentList = mountEquipmentMapper.selectList(queryWrapper);
        List<TauvMountEquipmentListVo> mountEquipmentListVoList = new ArrayList<>();
        if (!mountEquipmentList.isEmpty()) {
            mountEquipmentList.forEach(item -> {
                TauvMountEquipmentListVo mountEquipmentListVo = new TauvMountEquipmentListVo();
                // 拷贝属性
                BeanUtils.copyProperties(item, mountEquipmentListVo);
                // 类型描述
                mountEquipmentListVo.setTypeName(CommonConstant.MOUNT_EQUIPMENT_TYPE_LIST.get(item.getType()));
                // 状态描述
                mountEquipmentListVo.setStatusName(CommonConstant.MOUNT_EQUIPMENT_STATUS_LIST.get(item.getStatus()));
                mountEquipmentListVoList.add(mountEquipmentListVo);
            });
        }
        return response.success(mountEquipmentListVoList);
    }
}
