package com.taauav.admin.validate;

import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.Data;
import org.hibernate.validator.constraints.Length;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.DecimalMin;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.Date;

/**
 * @author dyg
 * @date 2019/11/22
 * @desc 执行任务数据验证
 */
@Data
public class ExecuteTask {
    @NotNull(message = "任务编号不能为空")
    private Integer id;

    /**
     * 执行时间
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date executionTime;

    @Length(min = 1, max = 20, message = "天气为1-20个字符")
    @NotBlank(message = "天气不能为空")
    private String weather;

    @Length(min = 1, max = 20, message = "风向为1-20个字符")
    @NotBlank(message = "风向不能为空")
    private String wind;

    @Length(min = 1, max = 20, message = "温度为1-20个字符")
    @NotBlank(message = "温度不能为空")
    private String temperature;

    @Length(min = 1, max = 20, message = "飞行速度为1-20个字符")
    @DecimalMin(value = "0", message = "飞行速度只能输入合理的数字")
    @NotBlank(message = "飞行速度不能为空")
    private String flightSpeed;

    @Length(min = 1, max = 20, message = "飞行高度为1-20个字符")
    @DecimalMin(value = "0", message = "飞行高度只能输入合理的数字")
    @NotBlank(message = "飞行高度不能为空")
    private String flightHeight;

    @Length(min = 1, max = 20, message = "飞行时间为1-20个字符")
    @DecimalMin(value = "0", message = "飞行时间只能输入合理的数字")
    @NotBlank(message = "飞行时间不能为空")
    private String flightTime;

    @Length(max = 200, message = "备注不能超过200个字符")
    private String remark;
}
