package com.taauav.common.config;

import com.taauav.admin.entity.SysAdmin;
import com.taauav.admin.entity.SysAuthRule;
import com.taauav.admin.service.ISysAdminService;
import com.taauav.admin.service.ISysAuthRuleService;
import com.taauav.common.util.ShiroUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.authc.*;
import org.apache.shiro.authz.AuthorizationInfo;
import org.apache.shiro.authz.SimpleAuthorizationInfo;
import org.apache.shiro.realm.AuthorizingRealm;
import org.apache.shiro.subject.PrincipalCollection;
import org.apache.shiro.util.ByteSource;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;

/**
 * @author ：dyg
 * @date ：2019/11/05 16:31
 * @description：自定义 shiroRealm, 主要是重写其认证、授权
 */
@Slf4j
public class MyShiroRealm extends AuthorizingRealm {

    @Resource
    private ISysAdminService iSysAdminService;
    @Resource
    private ISysAuthRuleService iSysAuthRuleService;
    /**
     * create by: dyg
     * description: 授权
     * create time: 2019/7/1 10:32
     *
     * @return 权限信息，包括角色以及权限
     */
    @Override
    protected AuthorizationInfo doGetAuthorizationInfo(PrincipalCollection principals) {
        log.warn("开始执行授权操作.......");

        SimpleAuthorizationInfo authorizationInfo = new SimpleAuthorizationInfo();
        //如果身份认证的时候没有传入User对象，这里只能取到userName
        //也就是SimpleAuthenticationInfo构造的时候第一个参数传递需要User对象
        Integer adminId = ShiroUtils.getAdminId();
        if (adminId.equals(1)) {
            //管理员拥有所有权限
            authorizationInfo.addStringPermission("*:*:*");
        } else {
            List<SysAuthRule> list = iSysAuthRuleService.getAuthRules();
            if (list != null) {
                for (SysAuthRule authRule : list) {
                    if (StringUtils.isEmpty(authRule.getPermission())) {
                        continue;
                    }
                    authorizationInfo.addStringPermission(authRule.getPermission());
                }
            }
        }

        return authorizationInfo;
    }

    /**
     * create by: dyg
     * description: 主要是用来进行身份认证的，也就是说验证用户输入的账号和密码是否正确。
     * create time: 2019/7/1 09:04
     *
     * @return 身份验证信息
     */
    @Override
    protected AuthenticationInfo doGetAuthenticationInfo(AuthenticationToken token) throws AuthenticationException {
        log.warn("开始进行身份认证......");

        //获取用户的输入的账号.
        String userName = (String) token.getPrincipal();
        //通过username从数据库中查找 User对象.
        //实际项目中，这里可以根据实际情况做缓存，如果不做，Shiro自己也是有时间间隔机制，2分钟内不会重复执行该方法
        SysAdmin user = iSysAdminService.findByUsername(userName);
        if (Objects.isNull(user)) {
            throw new UnknownAccountException("用户名或密码错误");
        }
        String status = "1";
        if (!(status.equals(user.getStatus().toString()))) {
            throw new UnknownAccountException("账号已被锁定，请联系管理员!");
        }
        return new SimpleAuthenticationInfo(
                // 这里传入的是user对象，比对的是用户名，直接传入用户名也没错，但是在授权部分就需要自己重新从数据库里取权限
                user,
                // 密码
                user.getPassword(),
                // salt = username + salt
                ByteSource.Util.bytes(""),
                // realm name
                getName()
        );
    }

}
