package com.taauav.common.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.taauav.common.bean.Response;
import com.taauav.common.core.mps.BaseQuery;
import com.taauav.common.domain.BaseEntity;
import com.taauav.common.domain.Entity;
import com.taauav.common.service.IBaseService;
import com.taauav.common.util.DateUtil;
import com.taauav.common.util.ShiroUtils;
import org.apache.ibatis.logging.Log;
import org.apache.ibatis.logging.LogFactory;
import org.springframework.validation.annotation.Validated;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.time.LocalDateTime;

/**
 * 基础服务类实现
 *
 * @param <M>
 * @param <T>
 */
@Validated
public class BaseServiceImpl<M extends BaseMapper<T>, T extends BaseEntity> extends ServiceImpl<M, T> implements IBaseService<T> {

    protected Log log = LogFactory.getLog(getClass());

    private Class<T> modeClass;

    public BaseServiceImpl() {
        Type type = this.getClass().getGenericSuperclass();
        this.modeClass = (Class) ((ParameterizedType) type).getActualTypeArguments()[1];
    }

    @Override
    public boolean addData(Entity entity) {
        Integer adminId = ShiroUtils.getAdminId();
        entity.setCreateTime(DateUtil.now());
        entity.setUpdateTime(DateUtil.now());
        entity.setCreateUser(adminId);
        entity.setUpdateUser(adminId);
        entity.setMark(1);
        return super.save((T) entity);
    }

    @Override
    public boolean editData(Entity entity) {
        Integer adminId = ShiroUtils.getAdminId();
        entity.setUpdateTime(DateUtil.now());
        entity.setUpdateUser(adminId);
        return super.updateById((T) entity);
    }

    @Override
    public T getInfo(Integer id) {
        return getInfoByField("id", id);
    }

    @Override
    public T getInfoByField(String field, Object value) {
        if (null == field) {
            return null;
        }
        QueryWrapper<T> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(field, value);
        queryWrapper.eq("mark", 1);
        return getOne(queryWrapper);
    }

    @Override
    public boolean deleteByIds(Integer[] ids, Entity entity) {
        UpdateWrapper<T> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("mark", 0);
        updateWrapper.in("id", ids);
        return update(updateWrapper);
    }

    @Override
    public boolean deleteById(Integer id, Entity entity) {
        entity.setMark(0);
        UpdateWrapper<T> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", id);
        updateWrapper.set("mark", 0);
        return update(updateWrapper);
    }
}
