package com.taauav.common.util;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import springfox.documentation.builders.ApiInfoBuilder;
import springfox.documentation.builders.PathSelectors;
import springfox.documentation.builders.RequestHandlerSelectors;
import springfox.documentation.service.ApiInfo;
import springfox.documentation.service.Contact;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spring.web.plugins.Docket;
import springfox.documentation.swagger2.annotations.EnableSwagger2;

/**
 * <p>
 * @Description: swagger2的配置文件，这里可以配置swagger2的一些基本的内容，比如扫描的包等等
 * </p>
 *
 * @author matengfei
 * @since 2019-11-08
 */
@Configuration
@EnableSwagger2
public class Swagger2Util {

    @Bean
    public Docket createRestApi() {
        return new Docket(DocumentationType.SWAGGER_2)
                //调用apiInfo方法,创建一个ApiInfo实例,里面是展示在文档页面信息内容
                .apiInfo(apiInfo())
                .select()
                //控制暴露出去的路径下的实例
                //如果某个接口不想暴露,可以使用以下注解
                //@ApiIgnore 这样,该接口就不会暴露在 swagger2 的页面下
                .apis(RequestHandlerSelectors.basePackage("com.taauav.admin.controller"))
                .paths(PathSelectors.any())
                .build();
    }

    /**
     * @Description: 构建api文档的信息
     */
    private ApiInfo apiInfo() {
        return new ApiInfoBuilder()
                // 设置页面标题
                .title("拓恒航空多源数据管理平台")
                // 条款地址
                .termsOfServiceUrl("http://despairyoke.github.io/")
                .contact(new Contact("拓恒航空", "http://admin.taauav.net", "matengfei@taauav.com"))
                //描述
                .description("接口文档描述")
                // 定义版本号
                .version("1.0")
                .build();
    }

}
