package com.taauav.front.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.admin.entity.TauvInspectDriver;
import com.taauav.admin.entity.TauvQuestionOptions;
import com.taauav.admin.entity.TauvReport;
import com.taauav.admin.entity.TauvReportRemark;
import com.taauav.admin.mapper.TauvInspectDriverMapper;
import com.taauav.admin.service.*;
import com.taauav.common.bean.Response;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.DateUtil;
import com.taauav.common.util.StringUtils;
import com.taauav.front.mapper.LSReportMapper;
import com.taauav.front.query.LSReportQuery;
import com.taauav.front.service.ILSReportService;
import com.taauav.front.vo.LSReportInfoVo;
import com.taauav.front.vo.LSReportListVo;
import com.taauav.front.vo.LSReportQuestionFileListVo;
import com.taauav.front.vo.LSReportQuestionListVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * <p>
 * 巡检报告表 服务实现类
 * </p>
 *
 * @author dyg
 * @since 2020-04-10
 */
@Service
public class LSReportServiceImpl extends BaseServiceImpl<LSReportMapper, TauvReport> implements ILSReportService {

    @Value("${server.UPLOAD_URL}")
    private String uploadUrl;

    @Autowired
    private LSReportMapper reportMapper;

    @Autowired
    private Response response;

    @Autowired
    private ISysCityService cityService;

    @Autowired
    private TauvInspectDriverMapper inspectDriverMapper;

    @Autowired
    private ITauvQuestionOptionsService questionOptionsService;

    @Autowired
    private ITauvInspectFileService inspectFileService;

    @Autowired
    private ITauvReportRemarkService reportRemarkService;

    @Autowired
    private ITauvReportEvaluationService reportEvaluationService;

    /**
     * 获取报告列表
     *
     * @param query 查询条件
     * @return
     */
    @Override
    public Response getList(LSReportQuery query) {
        // 查询条件
        QueryWrapper<TauvReport> queryWrapper = new QueryWrapper<>();
        // 报告编号
        if (!StringUtils.isEmpty(query.getReportNo())) {
            queryWrapper.like("report_no", query.getReportNo());
        }
        // 河湖名称
        if (!StringUtils.isEmpty(query.getDriverName())) {
            queryWrapper.like("driver_name", query.getDriverName());
        }
        // 巡检日期
        if (query.getInspectTime() != null) {
            String inspectTime = DateUtil.format(query.getInspectTime(), "yyyy-MM-dd");
            Date startTime = DateUtil.parse(inspectTime + " 0:00:00", "yyyy-MM-dd HH:mm:ss");
            Date endTime = DateUtil.parse(inspectTime + " 23:59:59", "yyyy-MM-dd HH:mm:ss");
        }

        // 区划ID
        if (query.getDriverArea() != null) {
            queryWrapper.eq("driver_area", query.getDriverArea());
        }
        queryWrapper.eq("mark", 1);

        // 查询数据
        IPage<TauvReport> page = new Page<>(query.getPage(), query.getPageSize());
        IPage<TauvReport> data = reportMapper.selectPage(page, queryWrapper);
        List<TauvReport> reportList = data.getRecords();
        List<LSReportListVo> reportListVoList = new ArrayList<>();
        if (!reportList.isEmpty()) {
            reportList.forEach(item -> {
                LSReportListVo reportListVo = new LSReportListVo();
                // 拷贝属性
                BeanUtils.copyProperties(item, reportListVo);
                // 区划
                String cityName = cityService.getCityName(item.getDriverArea());
                reportListVo.setDriverAreaName(cityName);
                // 状态描述
                reportListVo.setStatusName(item.getStatusList().get(item.getStatus()));
                reportListVoList.add(reportListVo);
            });
        }
        // 返回结果
        Map<String, Object> result = new HashMap<>();
        result.put("total", data.getTotal());
        result.put("size", data.getSize());
        result.put("current", data.getCurrent());
        result.put("pages", data.getPages());
        result.put("records", reportListVoList);
        return response.success(result);
    }

    /**
     * 获取报告详情
     *
     * @param id 报告ID
     * @return
     */
    @Override
    public Response detail(Integer id) {
        if (id == null || id <= 0) {
            return response.failure("报告ID不能为空");
        }
        // 获取报告信息
        TauvReport report = reportMapper.selectById(id);
        if (report == null) {
            return response.failure("报告信息不存在");
        }
        // 拷贝属性
        LSReportInfoVo reportInfoVo = new LSReportInfoVo();
        BeanUtils.copyProperties(report, reportInfoVo);
        // 区划描述
        String cityName = cityService.getCityName(reportInfoVo.getDriverArea());
        reportInfoVo.setDriverAreaName(cityName);
        // 报告状态描述
        reportInfoVo.setStatusName(report.getStatusList().get(report.getStatus()));

        // 获取巡检信息
        TauvInspectDriver inspectDriver = inspectDriverMapper.selectById(report.getInspectDriverId());
        if (inspectDriver != null) {
            reportInfoVo.setWeather(inspectDriver.getWeather());
            reportInfoVo.setWind(inspectDriver.getWind());
            reportInfoVo.setTemperature(inspectDriver.getTemperature());
            reportInfoVo.setFlightSpeed(inspectDriver.getFlightSpeed().toString());
            reportInfoVo.setFlightHeight(inspectDriver.getFlightHeight().toString());
            reportInfoVo.setFlightTime(inspectDriver.getFlightTime().toString());
        }

        TauvQuestionOptions question = new TauvQuestionOptions();
        // 分类列表
        Map<Integer, String> categoryList = question.getCategoryList();
        // 类型列表
        Map<Integer, String> typeList = question.getTypeList();
        // 获取问题列表
        List<TauvQuestionOptions> questionData = questionOptionsService.getDataList();
        List<LSReportQuestionListVo> reportQuestionListVoList = new ArrayList<>();
        if (!questionData.isEmpty()) {
            questionData.forEach(item -> {
                // 问题数量
                Integer questionNum = inspectFileService.getReviewNumByInspectDriverId(item.getId(), report.getInspectDriverId());
                // 报告备注
                TauvReportRemark remarkInfo = reportRemarkService.getInfoByReportIdAndQuestionId(report.getId(), report.getId());
                if (remarkInfo != null) {
                    LSReportQuestionListVo reportQuestionListVo = new LSReportQuestionListVo();
                    reportQuestionListVo.setId(item.getId());
                    reportQuestionListVo.setCategory(item.getCategory());
                    reportQuestionListVo.setCategoryName(categoryList.get(item.getCategory()));
                    reportQuestionListVo.setType(item.getType());
                    reportQuestionListVo.setTypeName(typeList.get(item.getType()));
                    reportQuestionListVo.setQuestionNum(questionNum);
                    reportQuestionListVo.setRemark(remarkInfo.getRemark());
                    reportQuestionListVo.setContent(item.getContent());
                    reportQuestionListVo.setNote(item.getNote());
                    reportQuestionListVo.setExamineScore(remarkInfo.getExamineScore());
                    reportQuestionListVo.setCheckScore(remarkInfo.getCheckScore());
                    reportQuestionListVoList.add(reportQuestionListVo);
                }
            });
        }
        // 加入报告
        reportInfoVo.setQuestionList(reportQuestionListVoList);

        // 报告描述
        List<Map<String, Object>> reportDescList = reportEvaluationService.getDataByStatus(1);
        reportInfoVo.setReportDescList(reportDescList);

        // 获取问题清单
        List<Map<String, Object>> questionFileList = inspectFileService.getReviewDataByInspectDriverId(report.getInspectDriverId());
        List<LSReportQuestionFileListVo> questionFileListVoList = new ArrayList<>();
        if (!questionFileList.isEmpty()) {
            questionFileList.forEach(item -> {
                // Map转对象
                LSReportQuestionFileListVo questionFileListVo = JSON.parseObject(JSON.toJSONString(item), LSReportQuestionFileListVo.class);
                if (questionFileListVo.getStatus() == 1) {
                    Integer questionId = questionFileListVo.getQuestionId();
                    if (questionId != null && questionId > 0) {
                        TauvQuestionOptions questionOptions = questionOptionsService.getInfoById(Integer.valueOf(questionId));
                        questionFileListVo.setQuestionContent(questionOptions.getContent());
                    }
                }
                // 图片地址
                if (!StringUtils.isEmpty(item.get("src").toString())) {
                    questionFileListVo.setFormatSrc(uploadUrl + item.get("src"));
                }
                // 原始图片
                if (!StringUtils.isEmpty(item.get("original_img").toString())) {
                    questionFileListVo.setFormatOriginalImg(uploadUrl + item.get("original_img").toString());
                }
                questionFileListVoList.add(questionFileListVo);
            });
        }
        // 设置问题清单
        reportInfoVo.setQuestionFileList(questionFileListVoList);
        return response.success(reportInfoVo);
    }
}
